//
// Created by Orest Savchak on 8/11/17.
// Copyright (c) 2017 GetSocial BV. All rights reserved.
//

#import <Foundation/Foundation.h>

/*!
 * You can check examples about Injection usage under Example directory.
 */

@interface GetSocialInjection : NSObject

+ (void)inject:(id)object dependencies:(NSSet *)dependencies;
+ (NSSet *)dependenciesForClass:(Class)clazz fromSelectors:(NSSet *)selectors;

@end

/*!
 * Macro defines which dependencies should be injected.
 */
#define getsocial_inject(dependencies...)                                                          \
    +(NSSet *)getsocial_dependencies                                                               \
    {                                                                                              \
        SEL selectors[] = {dependencies};                                                          \
        NSMutableSet *selectorNames = [NSMutableSet set];                                          \
        for (NSUInteger j = 0; j < sizeof(selectors) / sizeof(SEL); j++)                           \
        {                                                                                          \
            SEL selector = selectors[j];                                                           \
            [selectorNames addObject:NSStringFromSelector(selector)];                              \
        }                                                                                          \
        return [GetSocialInjection dependenciesForClass:[self class] fromSelectors:selectorNames]; \
    }

/*!
 * Macro initiates injection in current object. Usually is used in constructor.
 */
#define getsocial_make_injection() [GetSocialInjection inject:self dependencies:[[self class] getsocial_dependencies]];

/*!
 * Macro defines default constructor with injection - use it, if you don't need anything specific to be done in constructor.
 */
#define getsocial_injection_initializer() \
    -(instancetype)init                   \
    {                                     \
        if (self = [super init])          \
        {                                 \
            getsocial_make_injection();   \
        }                                 \
        return self;                      \
    }

#define getsocial_injection_ui()    \
    -(void)awakeFromNib             \
    {                               \
        getsocial_make_injection(); \
        [super awakeFromNib];       \
    }

#define getsocial_injectable_ui(dependencies...) getsocial_inject(dependencies) getsocial_injection_ui()

/*!
 * Macro defines dependencies, that should be injected and default constructor with injection.
 * Use this if you don't have anything specific to de done in constructor.
 */
#define getsocial_injectable_constructor(dependencies...) getsocial_inject(dependencies) getsocial_injection_initializer()
